/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef G_RECORDER_H
#define G_RECORDER_H


#ifdef __APPLE__  
	#include <stdint.h>
#else
	#include <cstdint>
#endif
#include <vector>
#include <pthread.h>


class Channel;


namespace giada {
namespace m {
namespace recorder
{


struct action
{
	int      chan;    
 	int      type;
	int      frame;   
	float    fValue;  
	uint32_t iValue;  
};



extern std::vector<int> frames;



extern std::vector<std::vector<action*>> global;

extern bool active;
extern bool sortedActions;   



void init();



bool hasActions(int chanIndex);



bool canRec(Channel *ch, bool clockRunning, bool mixerRecording);



void rec(int chan, int action, int frame, uint32_t iValue=0, float fValue=0.0f);



void clearChan(int chan);



void clearAction(int chan, char action);



void deleteAction(int chan, int frame, char type, bool checkValues,
  pthread_mutex_t *mixerMutex, uint32_t iValue=0, float fValue=0.0);



void deleteActions(int chan, int frame_a, int frame_b, char type,
  pthread_mutex_t *mixerMutex);



void clearAll();



void optimize();



void sortActions();



void updateBpm(float oldval, float newval, int oldquanto);



void updateSamplerate(int systemRate, int patchRate);

void expand(int old_fpb, int new_fpb);
void shrink(int new_fpb);



int getNextAction(int chan, char action, int frame, struct action **out,
	uint32_t iValue=0);



int getAction(int chan, char action, int frame, struct action **out);



void startOverdub(int chan, char action, int frame, unsigned bufferSize);
void stopOverdub(int currentFrame, int totalFrames, pthread_mutex_t *mixerMutex);

}}}; 

#endif
