/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef G_WAVE_H
#define G_WAVE_H


#include <sndfile.h>
#include <string>
#include "const.h"


class Wave
{
private:

	float* m_data;
	int m_size;		    
	int m_channels;
	int m_rate;
	int m_bits;
	bool m_logical;   
	bool m_edited;    
	
	std::string m_path; 

public:

	Wave();
	Wave(float* data, int size, int channels, int rate, int bits, const std::string& path);
	~Wave();
	Wave(const Wave& other);

	void setRate(int v);
	void setChannels(int v);
	void setData(float* data, int size);
	void setLogical(bool l);
	void setEdited(bool e);

	
	 
	void setPath(const std::string& p, int id=-1);

	std::string getBasename(bool ext=false) const;
	std::string getExtension() const;
	int getRate() const;
	int getChannels() const;
	std::string getPath() const;	
	int getBits() const;
	float* getData() const;
	int getSize() const;        
	int getDuration() const;
	bool isLogical() const;
	bool isEdited() const;

	

	void clear();

	

	void free();

	

	float* getFrame(int f) const;

};

#endif
