/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef GE_KEYBOARD_H
#define GE_KEYBOARD_H


#include <vector>
#include <FL/Fl_Scroll.H>
#include "../../../../core/const.h"


class Channel;
class geButton;
class geColumn;
class geChannel;
class geSampleChannel;


class geKeyboard : public Fl_Scroll
{
private:

	static const int COLUMN_GAP = 16;

	

	void refreshColIndexes();

	static void cb_addColumn  (Fl_Widget* v, void* p);
	inline void __cb_addColumn(int width=G_DEFAULT_COLUMN_WIDTH);

	bool bckspcPressed;
	bool endPressed;
	bool spacePressed;
	bool enterPressed;

	

	static int indexColumn;

	geButton* addColumnBtn;

	

	std::vector<geColumn*> columns;

public:

	geKeyboard(int X, int Y, int W, int H);

	int handle(int e);

	

	void init();

	

	geChannel* addChannel(int column, Channel* ch, int size, bool build=false);

	

	void addColumn(int width=380);

	

	void deleteChannel(geChannel* gch);

	

	void freeChannel(geChannel* gch);

	

	void updateChannel(geChannel* gch);

	

	void organizeColumns();

	

	void refreshColumns();

	

	geColumn* getColumnByIndex(int index);

	

	geColumn* getColumn(int i);

	

	void clear();

	

	void setChannelWithActions(geSampleChannel* gch);

	

	void printChannelMessage(int res);

	

	unsigned getTotalColumns() { return columns.size(); }

	

	int getColumnWidth(int i);
};


#endif
